/*
 * grasp.h
 *
 *  Created on: 14.12.2010
 *      Author: akasper
 */

#ifndef GRASP_H_
#define GRASP_H_

namespace OMDB { namespace Grasp {

struct CGrasp
{
	struct Pose
	{
		Pose()
		: x(0.f), y(0.f), z(0.f), roll(0.f), pitch(0.f), yaw(0.f)
		{}

		Pose( double dx, double dy, double dz, double droll, double dpitch, double dyaw )
		: x(dx), y(dy), z(dz), roll(droll), pitch(dpitch), yaw(dyaw)
		{}

		double x;
		double y;
		double z;
		double roll;
		double pitch;
		double yaw;
	};

	struct Joints
	{
		Joints()
		: ThumbAngle(0.f), ThumbBase(0.f), ThumbProximal(0.f), ThumbDistal(0.f),
		  FirstFingerBase(0.f), FirstFingerProximal(0.f), FirstFingerDistal(0.f),
		  MiddleFingerBase(0.f), MiddleFingerProximal(0.f), MiddleFingerDistal(0.f),
		  RingFingerBase(0.f), RingFingerProximal(0.f), RingFingerDistal(0.f),
		  LittleFingerBase(0.f), LittleFingerProximal(0.f), LittleFingerDistal(0.f)
		{}

		double ThumbAngle, ThumbBase, ThumbProximal, ThumbDistal;
		double FirstFingerBase, FirstFingerProximal, FirstFingerDistal;
		double MiddleFingerBase, MiddleFingerProximal, MiddleFingerDistal;
		double RingFingerBase, RingFingerProximal, RingFingerDistal;
		double LittleFingerBase, LittleFingerProximal, LittleFingerDistal;
	};

	CGrasp( double Quality ) :
		mQuality( Quality )
	{}

	double mQuality;
	Pose mApproachPose;
	Pose mGraspPose;

	Joints mApproachJoints;
	Joints mGraspJoints;
	Joints mGraspOptimalJoints;
};

} // Grasp
} // OMDB

namespace {
template<typename T>
T& operator<<( T& lhs, const OMDB::Grasp::CGrasp& rhs )
{
	lhs << "--------------------" << std::endl;
	lhs << "Grasp" << std::endl;
	lhs << "Quality: " << rhs.mQuality << std::endl;
	lhs << "ApproachPose: " << std::endl;
	lhs << rhs.mApproachPose;
	lhs << "GraspPose: " << std::endl;
	lhs << rhs.mGraspPose;
	lhs << "ApproachJoints: " << std::endl;
	lhs << rhs.mApproachJoints;
	lhs << "GraspJoints: " << std::endl;
	lhs << rhs.mGraspJoints;
	lhs << "GraspOptimalJoints: " << std::endl;
	lhs << rhs.mGraspOptimalJoints;

	return lhs;
}

template<typename T>
T& operator<<( T& lhs, const OMDB::Grasp::CGrasp::Pose& rhs )
{
	lhs << "Position: " << rhs.x << "/" << rhs.y << "/" << rhs.z << std::endl;
	lhs << "Orientation: " << rhs.roll << "/" << rhs.pitch << "/" << rhs.yaw << std::endl;
	return lhs;
}

template<typename T>
T& operator<<( T& lhs, const OMDB::Grasp::CGrasp::Joints& rhs )
{
	lhs << "ThumbAngle: " << rhs.ThumbAngle << std::endl;
	lhs << "ThumbBase: " << rhs.ThumbBase << std::endl;
	lhs << "ThumbProximal: " << rhs.ThumbProximal << std::endl;
	lhs << "ThumbDistal: " << rhs.ThumbDistal << std::endl;

	lhs << "FirstFingerBase" << rhs.FirstFingerBase << std::endl;
	lhs << "FirstFingerProximal" << rhs.FirstFingerProximal << std::endl;
	lhs << "FirstFingerDistal" << rhs.FirstFingerDistal << std::endl;

	lhs << "MiddleFingerBase" << rhs.MiddleFingerBase << std::endl;
	lhs << "MiddleFingerProximal" << rhs.MiddleFingerProximal << std::endl;
	lhs << "MiddleFingerDistal" << rhs.MiddleFingerDistal << std::endl;

	lhs << "RingFingerBase" << rhs.RingFingerBase << std::endl;
	lhs << "RingFingerProximal" << rhs.RingFingerProximal << std::endl;
	lhs << "RingFingerDistal" << rhs.RingFingerDistal << std::endl;

	lhs << "LittleFingerBase" << rhs.LittleFingerBase << std::endl;
	lhs << "LittleFingerProximal" << rhs.LittleFingerProximal << std::endl;
	lhs << "LittleFingerDistal" << rhs.LittleFingerDistal << std::endl;

	return lhs;
}

}

#endif /* GRASP_H_ */
