#include <fstream>
#include <sstream>
#include <cstring>

#include "grasp_xml_reader.h"

namespace {

void ReadJoints( rapidxml::xml_node<>* JointNode, OMDB::Grasp::CGrasp::Joints& Joints )
{
	std::stringstream S;
	S.str( std::string( JointNode->first_node( "ThumbAngle" )->value() ) );
	S >> Joints.ThumbAngle;
	S.clear();
	S.str( std::string( JointNode->first_node( "ThumbBase" )->value() ) );
	S >> Joints.ThumbBase;
	S.clear();
	S.str( std::string( JointNode->first_node( "ThumbProximal" )->value() ) );
	S >> Joints.ThumbProximal;
	S.clear();
	S.str( std::string( JointNode->first_node( "ThumbDistal" )->value() ) );
	S >> Joints.ThumbDistal;

	S.clear();
	S.str( std::string( JointNode->first_node( "FirstFingerBase" )->value() ) );
	S >> Joints.FirstFingerBase;
	S.clear();
	S.str( std::string( JointNode->first_node( "FirstFingerProximal" )->value() ) );
	S >> Joints.FirstFingerProximal;
	S.clear();
	S.str( std::string( JointNode->first_node( "FirstFingerDistal" )->value() ) );
	S >> Joints.FirstFingerDistal;

	S.clear();
	S.str( std::string( JointNode->first_node( "MiddleFingerBase" )->value() ) );
	S >> Joints.MiddleFingerBase;
	S.clear();
	S.str( std::string( JointNode->first_node( "MiddleFingerProximal" )->value() ) );
	S >> Joints.MiddleFingerProximal;
	S.clear();
	S.str( std::string( JointNode->first_node( "MiddleFingerDistal" )->value() ) );
	S >> Joints.MiddleFingerDistal;

	S.clear();
	S.str( std::string( JointNode->first_node( "RingFingerBase" )->value() ) );
	S >> Joints.RingFingerBase;
	S.clear();
	S.str( std::string( JointNode->first_node( "RingFingerProximal" )->value() ) );
	S >> Joints.RingFingerProximal;
	S.clear();
	S.str( std::string( JointNode->first_node( "RingFingerDistal" )->value() ) );
	S >> Joints.RingFingerDistal;

	S.clear();
	S.str( std::string( JointNode->first_node( "LittleFingerBase" )->value() ) );
	S >> Joints.LittleFingerBase;
	S.clear();
	S.str( std::string( JointNode->first_node( "LittleFingerProximal" )->value() ) );
	S >> Joints.LittleFingerProximal;
	S.clear();
	S.str( std::string( JointNode->first_node( "LittleFingerDistal" )->value() ) );
	S >> Joints.LittleFingerDistal;
}

void ReadPose( rapidxml::xml_node<>* PoseNode, OMDB::Grasp::CGrasp::Pose& Pose )
{
	std::stringstream S;
	S.str( std::string( PoseNode->first_node( "PositionX" )->value() ) );
	S >> Pose.x;
	S.clear();
	S.str( std::string( PoseNode->first_node( "PositionY" )->value() ) );
	S >> Pose.y;
	S.clear();
	S.str( std::string( PoseNode->first_node( "PositionZ" )->value() ) );
	S >> Pose.z;

	S.clear();
	S.str( std::string( PoseNode->first_node( "Roll" )->value() ) );
	S >> Pose.roll;
	S.clear();
	S.str( std::string( PoseNode->first_node( "Pitch" )->value() ) );
	S >> Pose.pitch;
	S.clear();
	S.str( std::string( PoseNode->first_node( "Yaw" )->value() ) );
	S >> Pose.yaw;
}

}

namespace OMDB { namespace Grasp {

CGraspReader::CGraspReader( const std::string& iFilePath )
{
	std::ifstream iFile( iFilePath.c_str() );

	// Get length of file for possible
	iFile.seekg( 0, std::ios::end );
	std::streampos Length = iFile.tellg();
	iFile.seekg( 0, std::ios::beg );

	std::vector<char> Buffer( static_cast<unsigned int>( Length ) );
	iFile.read( &Buffer[0], Length );
	std::string Source;
	Source.assign( Buffer.begin(), Buffer.end() );
	iFile.close();

	try {
		mXMLDocument.parse<0>( const_cast<char*>( Source.c_str() ) );
		Read();
	}
	catch( rapidxml::parse_error& e )
	{
		char* w = e.where<char>();
		std::string t = Source.substr( 0, (w - Source.c_str())/sizeof(char) -1 );
		int c = 1;
		std::string::size_type i = t.find_first_of( "\n" );
		while( i != std::string::npos )
		{
			i = t.find_first_of( "\n", i+1 );
			c++;
		}
			
		std::cerr << "Error parsing XML file: " << std::string( e.what() ) << " in line " << c << std::endl;
	}
}

CGraspReader::~CGraspReader()
{
}

bool CGraspReader::GetGrasp( int Index, CGrasp& oGrasp ) const
{
	if( Index >= 0 || Index < static_cast<int>( mGraspList.size() ) )
	{
		oGrasp = mGraspList.at( Index );
		return true;
	}
	else
	{
		return false;
	}
}

const std::string& CGraspReader::GetHandType() const
{
	return mHandType;
}

const std::string& CGraspReader::GetObject() const
{
	return mObject;
}

int CGraspReader::GetNumberOfGrasps() const
{
	return mNumberOfGrasps;
}

void CGraspReader::ReadApproachJoint( rapidxml::xml_node<>* GraspNode, CGrasp& Grasp )
{
	ReadJoints( GraspNode->first_node( "ApproachJoint" ), Grasp.mApproachJoints );
}

void CGraspReader::ReadApproachPose( rapidxml::xml_node<>* GraspNode, CGrasp& Grasp )
{
	ReadPose( GraspNode->first_node( "ApproachPose" ), Grasp.mApproachPose );
}

void CGraspReader::ReadGraspPose( rapidxml::xml_node<>* GraspNode, CGrasp& Grasp )
{
	ReadPose( GraspNode->first_node( "GraspPose" ), Grasp.mGraspPose );
}

void CGraspReader::ReadGraspJoint( rapidxml::xml_node<>* GraspNode, CGrasp& Grasp )
{
	ReadJoints( GraspNode->first_node( "GraspJoint" ), Grasp.mGraspJoints );
}

void CGraspReader::ReadGraspOptimalJoint( rapidxml::xml_node<>* GraspNode, CGrasp& Grasp )
{
	ReadJoints( GraspNode->first_node( "GraspOptimalJoint" ), Grasp.mGraspOptimalJoints );
}

void CGraspReader::Read()
{
	rapidxml::xml_node<>* GraspListNode = mXMLDocument.first_node( "GraspList" );
	rapidxml::xml_node<>* NumberNode = GraspListNode->first_node( "Number" );
	std::stringstream ss;
	ss << std::string( NumberNode->value() );
	// total number of grasps in this file
	ss >> mNumberOfGrasps;

	mHandType = std::string( GraspListNode->first_node( "Hand" )->value() );
	mObject = std::string( GraspListNode->first_node( "Object" )->value() );

	for( rapidxml::xml_node<>* I = GraspListNode->first_node( "Grasp" ); I; I = I->next_sibling( "Grasp" ) )
	{
		double Q;
		ss.clear();
		ss << std::string( I->first_attribute( "Quality" )->value() );
		ss >> Q;
		CGrasp CurrentGrasp( Q );
		ReadApproachPose( I, CurrentGrasp );
		ReadApproachJoint( I, CurrentGrasp );
		ReadGraspPose( I, CurrentGrasp );
		ReadGraspJoint( I, CurrentGrasp );
		ReadGraspOptimalJoint( I, CurrentGrasp );
		mGraspList.push_back( CurrentGrasp );
	}
}

} // Grasp
} // OMDB
