/*!\file
 *
 * \author  Matthias Engbarth, Alexander Kasper
 * \date    2010-05-04, 2010-12-15
 *
 */
//----------------------------------------------------------------------
#include "ImageDataReader.h"

#include <sstream>
#include <fstream>

namespace OMDB { namespace Image {

CImageDataReader::CImageDataReader( const std::string& FilePath )
{
	std::ifstream iFile( FilePath.c_str() );

	// Get length of file for possible
	iFile.seekg( 0, std::ios::end );
	std::streampos Length = iFile.tellg();
	iFile.seekg( 0, std::ios::beg );

	std::vector<char> Buffer( static_cast<unsigned int>( Length ) );
	iFile.read( &Buffer[0], Length );
	std::string Source;
	Source.assign( Buffer.begin(), Buffer.end() );
	iFile.close();

	rapidxml::xml_document<> XMLDoc;

	try {
		XMLDoc.parse<0>( const_cast<char*>( Source.c_str() ) );
		rapidxml::xml_node<>* ObjectNode = XMLDoc.first_node( "Object" );
		mImagesObject = CImagesObject( ObjectNode );
	}
	catch( rapidxml::parse_error& e )
	{
		char* w = e.where<char>();
		std::string t = Source.substr( 0, (w - Source.c_str())/sizeof(char) -1 );
		int c = 1;
		std::string::size_type i = t.find_first_of( "\n" );
		while( i != std::string::npos )
		{
			i = t.find_first_of( "\n", i+1 );
			c++;
		}
			
		std::cerr << "Error parsing XML file: " << std::string( e.what() ) << " in line " << c << std::endl;
	}
}

const CImagesObject& CImageDataReader::GetImagesObject() const
{
	return mImagesObject;
}

} // Image
} // OMDB
