/*!\file
 *
 * \author  Matthias Engbarth, Alexander Kasper
 * \date    2010-08-10, 2010-12-15
 *
 */

#include "ImageInfo.h"

#include <sstream>

namespace OMDB { namespace Image {


CImageInfo::CImageInfo( rapidxml::xml_node<>* Node )
{
	std::string Side( Node->first_attribute( "Camera" )->value() );
	if( Side == "left" )
		mCameraSide = false;
	else
		mCameraSide = true;

	double Helper;
	std::stringstream S;
	S << std::string( Node->first_attribute( "R1" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "R2" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "R3" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "R4" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "R5" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "R6" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "R7" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "R8" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "R9" )->value() );
	S >> Helper;
	mRotation.push_back( Helper );
	S.clear();

	S << std::string( Node->first_attribute( "TransX" )->value() );
	S >> Helper;
	mTranslation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "TransY" )->value() );
	S >> Helper;
	mTranslation.push_back( Helper );
	S.clear();
	S << std::string( Node->first_attribute( "TransZ" )->value() );
	S >> Helper;
	mTranslation.push_back( Helper );
	S.clear();

	S << std::string( Node->first_attribute( "LightFront" )->value() );
	S >> mLightFront;
	S.clear();
	S << std::string( Node->first_attribute( "LightBack" )->value() );
	S >> mLightBack;
	S.clear();
	S << std::string( Node->first_attribute( "LightCenter" )->value() );
	S >> mLightCenter;
	
	std::string FileName( Node->value() );
	FileName = FileName.substr( 0, FileName.find_last_of( '.' ) );
	mTIFFile = FileName + "tif";
	mPNGFile = FileName + "png";
}

const std::vector<double>& CImageInfo::GetCameraRotation() const
{
	return mRotation;
}

const std::vector<double>& CImageInfo::GetCameraTranslation() const
{
	return mTranslation;
}

const std::string& CImageInfo::GetTIFFile() const
{
	return mTIFFile;
}

const std::string& CImageInfo::GetPNGFile() const
{
	return mPNGFile;
}

const double CImageInfo::GetLightFront() const
{
	return mLightFront;
}

const double CImageInfo::GetLightBack() const
{
	return mLightBack;
}

const double CImageInfo::GetLightCenter() const
{
	return mLightCenter;
}

const bool CImageInfo::IsLeftCamera() const
{
	return !mCameraSide;
}

const bool CImageInfo::IsRightCamera() const
{
	return mCameraSide;
}

} // Image
} // OMDB
