//----------------------------------------------------------------------
/*!\file
 *
 * \author  Matthias Engbarth, Alexander Kasper
 * \date    2010-08-10, 2010-12-15
 */
//----------------------------------------------------------------------
#ifndef IMAGE_INFO_H
#define IMAGE_INFO_H
#include <iostream>
#include <string>
#include <vector>

#include <rapidxml.hpp>

namespace OMDB { namespace Image {

class CImageInfo
{
public:
	explicit CImageInfo( rapidxml::xml_node<>* Node );

	const std::vector<double>& GetCameraRotation() const;
	const std::vector<double>& GetCameraTranslation() const;

	const std::string& GetTIFFile() const;
	const std::string& GetPNGFile() const;

	const double GetLightFront() const;
	const double GetLightBack() const;
	const double GetLightCenter() const;

	const bool IsLeftCamera() const;
	const bool IsRightCamera() const;

private:
	/// <code>0</code> means left camera, <code>1</code> means right camera
	bool mCameraSide;

	std::string mTIFFile;
	std::string mPNGFile;

	/// contains R1-R9
	std::vector<double> mRotation;
	/// contains Tx, Ty, Tz
	std::vector<double> mTranslation;

	double mLightFront;
	double mLightBack;
	double mLightCenter;
};

} // Image
} // OMDB

namespace {

template<typename T>
T& operator<<( T& lhs, const OMDB::Image::CImageInfo& rhs )
{
	lhs << "ImageInfo:" << std::endl;
	if( rhs.IsLeftCamera() )
		lhs << "Camera: left" << std::endl;
	else
		lhs << "Camera: right" << std::endl;
	lhs << "CameraRotation: " << std::endl;
	int i = 1;
	std::vector<double> Rotation = rhs.GetCameraRotation();
	for( std::vector<double>::iterator I = Rotation.begin(); I != Rotation.end(); I++ )
	{
		lhs << "R" << i << " " << *I << " ";
		i++;
	}
	lhs << std::endl;
	lhs << "CameraTranslation: " << std::endl;
	std::vector<double> Translation = rhs.GetCameraTranslation();
	lhs << "X " << Translation[0] << " Y " << Translation[1] << " Z " << Translation[2] << std::endl;

	lhs << "Light front: " << rhs.GetLightFront();
	lhs << " Light back: " << rhs.GetLightBack();
	lhs << " Light center: " << rhs.GetLightCenter() << std::endl;

	lhs << "TIF file: " << rhs.GetTIFFile() << std::endl;
	lhs << "PNG file: " << rhs.GetPNGFile() << std::endl;

	return lhs;
}

}

#endif // IMAGE_INFO_H
