/*!\file
 *
 * \author  Matthias Engbarth, Alexander Kasper
 * \date    2010-08-10, 2010-12-15
 *
 */
//----------------------------------------------------------------------
#include "ImagesObject.h"

#include <sstream>

namespace OMDB { namespace Image {

CImagesObject::CImagesObject()
: mName("")
{
}

CImagesObject::CImagesObject( rapidxml::xml_node<>* ObjectNode )
{
	mName = std::string( ObjectNode->first_attribute( "Name" )->value() );
	std::string TRP( ObjectNode->first_attribute( "TranslationRotaryPlate" )->value() );
	std::stringstream S( TRP.substr( 0, TRP.find_first_of("/") ) );
	S >> mPlate[0];
	S.clear();
	S.str( TRP.substr( TRP.find_first_of("/") + 1, TRP.find_last_of("/") - TRP.find_first_of("/") - 1 ) );
	S >> mPlate[1];
	S.clear();
	S.str( TRP.substr( TRP.find_last_of("/") + 1 ) );
	S >> mPlate[2];

	rapidxml::xml_node<>* ImagesNode = ObjectNode->first_node( "Images" );
	
	for( rapidxml::xml_node<>* I = ImagesNode->first_node( "Image" ); I; I = I->next_sibling( "Image" ) )
	{
		mImages.push_back( CImageInfo( I ) );
	}
}

void CImagesObject::Clear()
{
  mName.clear();

  for (int i = 0; i < 3; i++)
  {
	mPlate[i] = 0.f;
  }

  mImages.clear();
}

const std::vector<CImageInfo>& CImagesObject::GetImages() const
{
	return mImages;
}

const std::string& CImagesObject::GetName() const
{
	return mName;
}

} // Image
} // OMDB
